@@ -60,24 +60,85 @@
 ;; applying ordering constraints.
 ;;
 (ns com.puppetlabs.puppetdb.query
-  (:require [clojure.string :as string])
-  (:use [com.puppetlabs.utils :only [parse-number]]
+  (:require [clojure.string :as string]
+            [clojure.set :as set])
+  (:use [com.puppetlabs.utils :only [parse-number keyset valset]]
         [com.puppetlabs.puppetdb.scf.storage :only [db-serialize sql-as-numeric sql-array-query-string sql-regexp-match sql-regexp-array-match]]
-        [com.puppetlabs.jdbc :only [valid-jdbc-query?]]
+        [com.puppetlabs.jdbc :only [valid-jdbc-query? limited-query-to-vec query-to-vec paged-sql count-sql get-result-count]]
+        [com.puppetlabs.puppetdb.query.paging :only [requires-paging?]]
         [clojure.core.match :only [match]]))
 
+(defn execute-paged-query*
+  "Helper function to executed paged queries.  Builds up the paged sql string,
+  executes the query, and returns map containing the `:result` key and an
+  optional `:count` key."
+  [fail-limit query {:keys [limit offset order-by count?] :as paging-options}]
+  {:pre [(and (integer? fail-limit) (>= fail-limit 0))
+         (valid-jdbc-query? query)
+         ((some-fn nil? integer?) limit)
+         ((some-fn nil? integer?) offset)
+         ((some-fn nil? sequential?) order-by)
+         (every? map? order-by)]
+   :post [(map? %)
+          (vector? (:result %))
+          ((some-fn nil? integer?) (:count %))]}
+  (let [[sql & params] (if (string? query) [query] query)
+        paged-sql      (paged-sql sql paging-options)
+        result         {:result
+                          (limited-query-to-vec
+                            fail-limit
+                            (apply vector paged-sql params))}]
+      ;; TODO: this could also be implemented using `COUNT(*) OVER()`,
+      ;; which would allow us to get the results and the count via a
+      ;; single query (rather than two separate ones).  Need to do
+      ;; some benchmarking to see which is faster.
+      (if count?
+        (assoc result :count
+          (get-result-count (apply vector (count-sql sql) params)))
+        result)))
+
+(defn execute-query*
+  "Helper function to executed non-paged queries.  Returns a map containing the
+  `:result` key."
+  [fail-limit query]
+  {:pre [(and (integer? fail-limit) (>= fail-limit 0))
+         (valid-jdbc-query? query)]
+   :post [(map? %)
+         (vector? (:result %))]}
+  {:result (limited-query-to-vec fail-limit query)})
+
+(defn execute-query
+  "Given a query and a map of paging options, adds the necessary SQL for
+  implementing the paging, executes the query, and returns a map containing
+  the results and metadata.
+
+  The return value will contain a key `:result`, whose value is a vector of
+  the query results.  If the paging options indicate that a 'total record
+  count' should be returned, then the map will also include a key `:count`,
+  whose value is an integer indicating the total number of results available."
+  ([query paging-options] (execute-query 0 query paging-options))
+  ([fail-limit query {:keys [limit offset order-by] :as paging-options}]
+   {:pre [((some-fn string? sequential?) query)]
+    :post [(map? %)
+           (vector? (:result %))
+           ((some-fn nil? integer?) (:count %))]}
+    (let [sql-and-params (if (string? query) [query] query)]
+      (if (requires-paging? paging-options)
+        (execute-paged-query* fail-limit sql-and-params paging-options)
+        (execute-query* fail-limit sql-and-params)))))
+
 (defn compile-term
   "Compile a single query term, using `ops` as the set of legal operators. This
   function basically just checks that the operator is known, and then
   dispatches to the function implementing it."
   [ops [op & args :as term]]
   (when-not (sequential? term)
-    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must be an array" term))))
+    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must be an array" (vec term)))))
   (when-not op
-    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must contain at least one operator" term))))
+    (throw (IllegalArgumentException. (format "%s is not well-formed: queries must contain at least one operator" (vec term)))))
   (if-let [f (ops op)]
     (apply f args)
-    (throw (IllegalArgumentException. (format "%s is not well-formed: query operator '%s' is unknown" term op)))))
+    (throw (IllegalArgumentException. (format "%s is not well-formed: query operator '%s' is unknown" (vec term) op)))))
 
 (defn compile-boolean-operator*
   "Compile a term for the boolean operator `op` (AND or OR) applied to
@@ -133,25 +194,72 @@
     (throw (IllegalArgumentException. (format "'not' takes exactly one argument, but %d were supplied" (count terms)))))
   (negate-term* ops (first terms)))
 
-(def fact-columns #{"certname" "name" "value"})
+;; This map's keys are the queryable fields for facts, and the values are the
+;;  corresponding table names where the fields reside
+(def fact-columns {"certname" "certname_facts"
+                   "name"     "certname_facts"
+                   "value"    "certname_facts"})
 
-(def resource-columns #{"certname" "catalog" "resource" "type" "title" "tags" "exported" "sourcefile" "sourceline"})
+;; This map's keys are the queryable fields for resources, and the values are the
+;;  corresponding table names where the fields reside
+(def resource-columns {"certname"   "certname_catalogs"
+                       "catalog"    "catalog_resources"
+                       "resource"   "catalog_resources"
+                       "type"       "catalog_resources"
+                       "title"      "catalog_resources"
+                       "tags"       "catalog_resources"
+                       "exported"   "catalog_resources"
+                       "file"       "catalog_resources"
+                       "line"       "catalog_resources"})
 
-(def node-columns #{"name" "deactivated"})
+;; This map's keys are the names of fields from the resource table that were
+;; renamed in v3 of the query API.  The values are the new names of the fields.
+(def v3-renamed-resource-columns {"sourcefile" "file"
+                                  "sourceline" "line"})
 
-(def selectable-columns
-  {:resource resource-columns
-   :fact fact-columns
-   :node node-columns})
+;; This map's keys are the queryable fields for nodes, and the values are the
+;;  corresponding table names where the fields reside
+(def node-columns {"name"         "certnames"
+                   "deactivated"  "certnames"})
+
+(defn column-map->sql
+  "Helper function that converts one of our column maps to a SQL string suitable
+  for use in a SELECT"
+  [col-map]
+  (string/join ", "
+    (for [[field table] col-map]
+      (str table "." field))))
+
+(defmulti queryable-fields
+  "This function takes a query type (:resource, :fact, :node) and a query
+   API version number, and returns a set of strings which are the names the
+   fields that are legal to query"
+  (fn [query-type query-api-version] query-type))
+
+(defmethod queryable-fields :resource
+  [_ query-api-version]
+  (condp = query-api-version
+    3 (keyset resource-columns)
+    2 (-> (keyset resource-columns)
+        (set/union (keyset v3-renamed-resource-columns))
+        (set/difference (valset v3-renamed-resource-columns)))))
+
+(defmethod queryable-fields :fact
+  [_ _]
+  (keyset fact-columns))
+
+(defmethod queryable-fields :node
+  [_ _]
+  (keyset node-columns))
 
 (def subquery->type
   {"select-resources" :resource
-   "select-facts" :fact})
+   "select-facts"     :fact})
 
 (defn compile-extract
   "Compile an `extract` operator, selecting the given `field` from the compiled
   result of `subquery`, which must be a kind of `select` operator."
-  [ops field subquery]
+  [query-api-version ops field subquery]
   {:pre [(string? field)
          (coll? subquery)]
    :post [(map? %)
@@ -160,8 +268,8 @@
         subquery-type (subquery->type (first subquery))]
     (when-not subquery-type
       (throw (IllegalArgumentException. (format "The argument to extract must be a select operator, not '%s'" (first subquery)))))
-    (when-not (get-in selectable-columns [subquery-type field])
-      (throw (IllegalArgumentException. (format "Can't extract unknown %s field '%s'. Acceptable fields are: %s" (name subquery-type) field (string/join ", " (sort (selectable-columns subquery-type)))))))
+    (when-not (get (queryable-fields subquery-type query-api-version) field)
+      (throw (IllegalArgumentException. (format "Can't extract unknown %s field '%s'. Acceptable fields are: %s" (name subquery-type) field (string/join ", " (sort (queryable-fields subquery-type query-api-version)))))))
     {:where (format "SELECT r1.%s FROM (%s) r1" field subselect)
      :params params}))
 
@@ -169,13 +277,13 @@
   "Compile an `in` operator, selecting rows for which the value of
   `field` appears in the result given by `subquery`, which must be an `extract`
   composed with a `select`."
-  [kind ops field subquery]
+  [kind query-api-version ops field subquery]
   {:pre [(string? field)
          (coll? subquery)]
    :post [(map? %)
           (string? (:where %))]}
-  (when-not (get-in selectable-columns [kind field])
-    (throw (IllegalArgumentException. (format "Can't match on unknown %s field '%s' for 'in'. Acceptable fields are: %s" (name kind) field (string/join ", " (sort (selectable-columns kind)))))))
+  (when-not (get (queryable-fields kind query-api-version) field)
+    (throw (IllegalArgumentException. (format "Can't match on unknown %s field '%s' for 'in'. Acceptable fields are: %s" (name kind) field (string/join ", " (sort (queryable-fields kind query-api-version)))))))
   (when-not (= (first subquery) "extract")
     (throw (IllegalArgumentException. (format "The subquery argument of 'in' must be an 'extract', not '%s'" (first subquery)))))
   (let [{:keys [where] :as compiled-subquery} (compile-term ops subquery)]
@@ -187,7 +295,7 @@
   [ops query]
   {:post [valid-jdbc-query? %]}
   (let [{:keys [where params]} (compile-term ops query)
-        sql (format "SELECT %s FROM catalog_resources JOIN certname_catalogs USING(catalog) WHERE %s" (string/join ", " resource-columns) where)]
+        sql (format "SELECT %s FROM catalog_resources JOIN certname_catalogs USING(catalog) WHERE %s" (column-map->sql resource-columns) where)]
     (apply vector sql params)))
 
 (defn fact-query->sql
@@ -196,7 +304,7 @@
   [ops query]
   {:post [valid-jdbc-query? %]}
   (let [{:keys [where params]} (compile-term ops query)
-        sql (format "SELECT %s FROM certname_facts WHERE %s" (string/join ", " (map #(str "certname_facts." %) fact-columns)) where)]
+        sql (format "SELECT %s FROM certname_facts WHERE %s" (column-map->sql fact-columns) where)]
     (apply vector sql params)))
 
 (defn node-query->sql
@@ -205,11 +313,11 @@
   [ops query]
   {:post [valid-jdbc-query? %]}
   (let [{:keys [where params]} (compile-term ops query)
-        sql (format "SELECT %s FROM certnames WHERE %s" (string/join ", " node-columns) where)]
+        sql (format "SELECT %s FROM certnames WHERE %s" (column-map->sql node-columns) where)]
     (apply vector sql params)))
 
-(defn compile-resource-equality-v2
-  "Compile an = operator for a v2 resource query. `path` represents the field
+(defn compile-resource-equality-v3
+  "Compile an = operator for a v3 resource query. `path` represents the field
   to query against, and `value` is the value."
   [& [path value :as args]]
   {:post [(map? %)
@@ -239,7 +347,7 @@
           :params [name (db-serialize value)]}
 
          ;; metadata match.
-         [(metadata :when #{"catalog" "resource" "type" "title" "tags" "exported" "sourcefile" "sourceline"})]
+         [(metadata :when #{"catalog" "resource" "type" "title" "tags" "exported" "file" "line"})]
            {:where  (format "catalog_resources.%s = ?" metadata)
             :params [value]}
 
@@ -247,6 +355,22 @@
          :else (throw (IllegalArgumentException.
                        (str path " is not a queryable object for resources")))))
 
+(defn compile-resource-equality-v2
+  "Compile an = operator for a v2 resource query. `path` represents the field
+  to query against, and `value` is the value. This mostly just defers to
+  `compile-resource-equality-v3`, with a little bit of logic to handle the one
+  term that differs."
+  [& [path value :as args]]
+  {:post [(map? %)
+          (:where %)]}
+  (when-not (= (count args) 2)
+    (throw (IllegalArgumentException. (format "= requires exactly two arguments, but %d were supplied" (count args)))))
+  ;; If they passed in any of the new names for the renamed resource-columns, we fail
+  ;; because this is v2.
+  (when (contains? (valset v3-renamed-resource-columns) path)
+    (throw (IllegalArgumentException. (format "%s is not a queryable object for resources" path))))
+  (compile-resource-equality-v3 (get v3-renamed-resource-columns path path) value))
+
 (defn compile-resource-equality-v1
   "Compile an = operator for a v1 resource query. `path` represents the field
   to query against, and `value` is the value. This mostly just defers to
@@ -265,8 +389,8 @@
   (let [path (if (= path ["node" "name"]) "certname" path)]
     (compile-resource-equality-v2 path value)))
 
-(defn compile-resource-regexp
-  "Compile an '~' predicate for a resource query, which does regexp matching.
+(defn compile-resource-regexp-v3
+  "Compile an '~' predicate for a v3 resource query, which does regexp matching.
   This is done by leveraging the correct database-specific regexp syntax to
   return only rows where the supplied `path` match the given `pattern`."
   [path pattern]
@@ -283,7 +407,7 @@
           :params [pattern]}
 
          ;; metadata match.
-         [(metadata :when #{"catalog" "resource" "type" "title" "exported" "sourcefile" "sourceline"})]
+         [(metadata :when #{"type" "title" "exported" "file"})]
          {:where  (sql-regexp-match (format "catalog_resources.%s" metadata))
           :params [pattern]}
 
@@ -291,6 +415,21 @@
          :else (throw (IllegalArgumentException.
                         (str path " cannot be the target of a regexp match")))))
 
+(defn compile-resource-regexp-v2
+  "Compile an '~' predicate for a v2 resource query, which does regexp matching.
+  This is done by leveraging the correct database-specific regexp syntax to
+  return only rows where the supplied `path` match the given `pattern`.  This
+  mostly just defers to `compile-resource-regexp-v2`, with a little bit of logic to
+  handle the terms that differ."
+  [& [path value :as args]]
+  {:post [(map? %)
+          (:where %)]}
+  ;; If they passed in any of the new names for the renamed resource-columns, we fail
+  ;; because this is v2.
+  (when (contains? (valset v3-renamed-resource-columns) path)
+    (throw (IllegalArgumentException. (format "%s cannot be the target of a regexp match" path))))
+  (compile-resource-regexp-v3 (get v3-renamed-resource-columns path path) value))
+
 (defn compile-fact-equality
   "Compile an = predicate for a fact query. `path` represents the field to
   query against, and `value` is the value."
@@ -458,16 +597,33 @@
   [op]
   (condp = (string/lower-case op)
     "=" compile-resource-equality-v2
-    "~" compile-resource-regexp
+    "~" compile-resource-regexp-v2
     "and" (partial compile-and resource-operators-v2)
     "or" (partial compile-or resource-operators-v2)
     "not" (partial compile-not-v2 resource-operators-v2)
-    "extract" (partial compile-extract resource-operators-v2)
-    "in" (partial compile-in :resource resource-operators-v2)
+    "extract" (partial compile-extract 2 resource-operators-v2)
+    "in" (partial compile-in :resource 2 resource-operators-v2)
     "select-resources" (partial resource-query->sql resource-operators-v2)
     "select-facts" (partial fact-query->sql fact-operators-v2)
     nil))
 
+(defn resource-operators-v3
+  "Maps v3 resource query operators to the functions implementing them. Returns nil
+  if the operator isn't known."
+  [op]
+  (condp = (string/lower-case op)
+    "=" compile-resource-equality-v3
+    "~" compile-resource-regexp-v3
+    "and" (partial compile-and resource-operators-v3)
+    "or" (partial compile-or resource-operators-v3)
+    "not" (partial compile-not-v2 resource-operators-v3)
+    "extract" (partial compile-extract 3 resource-operators-v3)
+    "in" (partial compile-in :resource 3 resource-operators-v3)
+    "select-resources" (partial resource-query->sql resource-operators-v3)
+    "select-facts" (partial fact-query->sql fact-operators-v2)
+    nil))
+
+
 (defn fact-operators-v2
   "Maps v2 fact query operators to the functions implementing them. Returns nil
   if the operator isn't known."
@@ -484,8 +640,8 @@
       (= op "and") (partial compile-and fact-operators-v2)
       (= op "or") (partial compile-or fact-operators-v2)
       (= op "not") (partial compile-not-v2 fact-operators-v2)
-      (= op "extract") (partial compile-extract fact-operators-v2)
-      (= op "in") (partial compile-in :fact fact-operators-v2)
+      (= op "extract") (partial compile-extract 2 fact-operators-v2)
+      (= op "in") (partial compile-in :fact 2 fact-operators-v2)
       ;; select-resources uses a different set of operators-v2, of course
       (= op "select-resources") (partial resource-query->sql resource-operators-v2)
       (= op "select-facts") (partial fact-query->sql fact-operators-v2))))
@@ -517,7 +673,7 @@
       (= op "and") (partial compile-and node-operators-v2)
       (= op "or") (partial compile-or node-operators-v2)
       (= op "not") (partial compile-not-v1 node-operators-v2)
-      (= op "extract") (partial compile-extract node-operators-v2)
-      (= op "in") (partial compile-in :node node-operators-v2)
+      (= op "extract") (partial compile-extract 2 node-operators-v2)
+      (= op "in") (partial compile-in :node 2 node-operators-v2)
       (= op "select-resources") (partial resource-query->sql resource-operators-v2)
       (= op "select-facts") (partial fact-query->sql fact-operators-v2))))
